/*
 * Created on 22 avr. 2004 by the Message Center Team
 *
 */

package mc.formgenerator.ggf;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Writer;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import mc.formgenerator.ggf.xslprocessing.XalanExecutor;

import org.chiba.tools.xslt.StylesheetLoader;
import org.chiba.tools.xslt.UIGenerator;
import org.chiba.tools.xslt.XSLTGenerator;
import org.chiba.xml.xforms.ChibaBean;
import org.chiba.xml.xforms.ChibaContext;
import org.chiba.xml.xforms.config.Config;
import org.chiba.xml.xforms.events.EventFactory;
import org.chiba.xml.xforms.events.XFormsEvent;
import org.chiba.xml.xforms.exception.XFormsException;
import org.chiba.xml.xforms.ui.Repeat;
import org.w3c.dom.Document;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventTarget;


/**
 * Implementation of the generic form generator
 * This class will be call by the servletGF
 * @author sempereb
 */
public class GGF implements org.w3c.dom.events.EventListener{

	//Xalan executor instance for xsl processing
	public XalanExecutor xalanExe;

	//Chiba bean for conversion
	public ChibaBean chibaBean;
	
	//Chiba UI generator
	public UIGenerator xformsTransfomer;
	
	//Context for the webapp
	public String contextPath = null;
	
	//Action URL
	public String actionURL = null;
	
	//Chiba context
	private ChibaContext webContext = null;
	
	//Parameters names
	private Map parameterNames = null;
	
	//processing-phase handling - signals to UIGenerator like ServletAdapter from Chiba
	private static final String DATA_PREFIX_PROPERTY = "chiba.web.dataPrefix";
	private static final String TRIGGER_PREFIX_PROPERTY = "chiba.web.triggerPrefix";
	private static final String SELECTOR_PREFIX_PROPERTY = "chiba.web.selectorPrefix";
	private static final String DATA_PREFIX_DEFAULT = "d_";
	private static final String TRIGGER_PREFIX_DEFAULT = "t_";
	private static final String SELECTOR_PREFIX_DEFAULT = "s_";
	private static final String INIT_PHASE = "init";
	private static final String SESSION_PHASE = "session";
	private static final String SUBMIT_PHASE = "submit";
	private String selectorPrefix = null;
	private String dataPrefix = null;
	private String triggerPrefix = null;
	private String phase; 
	

	/**
	 * Default constructor for our class 
	 * @param contextPath Context path for uri resolution
	 * @param actionURL URL for submission
	 */
	public GGF(String contextPath, String actionURL){
		
		//Setting the context path attribute
		this.contextPath = contextPath;
		
		//Setting the actionURL attribute
		this.actionURL = actionURL;
		
		//Instanciation of Xalan executor variable
	 	this.xalanExe = new XalanExecutor( this.contextPath + "\\WEB-INF\\xslt\\generator.xsl");
		
		//Instanciation of the chiba bean
		this.chibaBean = new ChibaBean();

		//Instanciation of the xforms transformer
		this.xformsTransfomer = null;
		
		//Parameters names
		this.parameterNames = new HashMap();
		
		//Setting chiba context
		this.webContext = new WebContextGGF(this);
		this.webContext.setProperty(WebContextGGF.REQUEST_PARAMETERS,this.parameterNames);
	}
	 
	
	
	
	/**
	 * Generates a xhtml dom thanks to the xsl file and the xml dom in input
	 * @param xmlInput: The xml file that will be submit to the xsl transformation
	 * @return a Document, a dom that represents the generated document
	 * @throws XFormsException
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws TransformerException
	 * @throws IOException
	 */
	private Document generateXforms (Document xmlInput) throws XFormsException, TransformerConfigurationException, FileNotFoundException, TransformerException, IOException{
		//Output Document
		Document xformsOutput;
		
		//Dom tranformation processing
		xformsOutput = this.xalanExe.XalanExecutionDOM(xmlInput);
 	
	 	//Return the Xforms document
	 	return xformsOutput;
	 }
	 
	 
	 
	 
	/**
	 * Transformer for interpretting xforms document
	 * @param stylesLoader
	 * @param cp
	 * @param actionURL
	 * @return
	 * @throws XFormsException
	 */
	private UIGenerator createTransformer(StylesheetLoader stylesLoader,
										  ChibaBean cp,
										  String actionURL)
			throws XFormsException {
		if (this.xformsTransfomer == null) {
			this.xformsTransfomer = new XSLTGenerator(stylesLoader);
		}
		this.xformsTransfomer.setParameter("action-url", actionURL);
		this.xformsTransfomer.setParameter("selector-prefix", getSelectorPrefix());
		this.xformsTransfomer.setParameter("debug-enabled", String.valueOf(true));

		return this.xformsTransfomer;
	}




	/**
	 * Initializes chiba in order to interprete the xforms document
	 * For this we have to inject the document in the chiba bean and init this last
	 * @param xformsDocument: The Xforms documents we have to interprete
	 * @throws XFormsException
	 */
	private void initChiba(Document xformsDocument, Document xmlInput) throws XFormsException{

		//Stylesheetpath
		String stylesheetPath = this.contextPath + "\\WEB-INF\\xslt";

		//Set the current context
		this.chibaBean.setContext(this.webContext);
		
		//Base uri
		this.chibaBean.setBaseURI(this.contextPath);
		
		//Injection of the xml document into the bean
		this.chibaBean.setXMLContainer(xformsDocument);
		
		//Setting the initial instance
		this.chibaBean.importInstance("instance", xmlInput.getDocumentElement());
		
		//create and configure StylesheetLoader
		StylesheetLoader stylesLoader = new StylesheetLoader(stylesheetPath);

		//create and configure User Interface generator and pass loader to it
		this.xformsTransfomer = createTransformer(stylesLoader, this.chibaBean, this.actionURL);
		
		//Attach listener that sets submit processing phase
		addPhaseListener(this.chibaBean);
		
		//Bean initialization
		this.chibaBean.init();
	}
	 
	 
	 
	 
	/**
	 * Initializes GGF 
	 * @param xmlInput: The document send to the xsl engine
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws XFormsException
	 * @throws TransformerException
	 * @throws IOException
	 */
	public void initGGF(Document xmlInput) throws TransformerConfigurationException, FileNotFoundException, XFormsException, TransformerException, IOException{
	 	//Step1 - xforms generation by xsl file and xsl engine
	 	Document xformsDocument = this.generateXforms(xmlInput);
	 	
	 	//Step2 - Chiba initialization
	 	if(xformsDocument != null){
	 		this.initChiba(xformsDocument, xmlInput);
	 	}
	 	else
	 		System.out.println("***Null generated Document by generateXforms method***");	 	
	}
	
	
	
	
	/**
	 *	Writing the form to user by using Chiba API 
	 */
	public void runGGF(Writer out) throws XFormsException{
		//Generetion of the output dom
		if(xformsTransfomer != null){
			this.buildUI(out);
		}
		else
			System.out.println("***Null XformsTransformer***");
	}




	/**
	 * Generates the user interface.
	 * @param responseWriter : The Writer to use for the result stream
	 * @throws XFormsException
	 */
	public void buildUI(Writer responseWriter) throws XFormsException {

		//Setting parameters to the transformer
		this.xformsTransfomer.setParameter("phase", this.phase);
		this.xformsTransfomer.setInputNode(this.chibaBean.getXMLContainer());
		
		//Generation of the user interface
		this.xformsTransfomer.generate(responseWriter);

		//Phase listenning
		if (this.phase.equals(INIT_PHASE)) {
			this.phase = SESSION_PHASE;
		}
	}
	
	
	
	
	/**
	 * This method is called whenever an event occurs of the type for which
	 * the <code> EventListener</code> interface was registered.
	 *
	 * @param evt The <code>Event</code> contains contextual information
	 *            about the event. It also contains the <code>stopPropagation</code>
	 *            and <code>preventDefault</code> methods which are used in
	 *            determining the event's flow and default action.
	 */
	public void handleEvent(Event evt) {
		if (evt instanceof XFormsEvent) {

			if (evt.getType().equals(EventFactory.READY)) {
				this.phase = INIT_PHASE;
			}

			if (evt.getType().equals(EventFactory.SUBMIT)) {
				this.phase = SUBMIT_PHASE;
			}
		}		
	}
	
	
	

	/**
	 * Returns the prefix which is used to identify data parameters.
	 * @return the prefix which is used to identify data parameters.
	 */
	public String getDataPrefix() {
		if (this.dataPrefix == null) {
			try {
				this.dataPrefix = Config.getInstance().getProperty(DATA_PREFIX_PROPERTY, DATA_PREFIX_DEFAULT);
			} catch (Exception e) {
				this.dataPrefix = DATA_PREFIX_DEFAULT;
			}
		}

		return this.dataPrefix;
	}
	
	
	

	/**
	 * Returns the prefix which is used to identify trigger parameters.
	 * @return the prefix which is used to identify trigger parameters
	 */
	public String getTriggerPrefix() {
		if (this.triggerPrefix == null) {
			try {
				this.triggerPrefix =
						Config.getInstance().getProperty(TRIGGER_PREFIX_PROPERTY, TRIGGER_PREFIX_DEFAULT);
			} catch (Exception e) {
				this.triggerPrefix = TRIGGER_PREFIX_DEFAULT;
			}
		}

		return this.triggerPrefix;
	}
	
	
	
	
	/**
	 * returns the configured prefix which identifies 'selector' parameters. These are used to transport
	 * the state of repeat indices via http.
	 *
	 * @return the prefix for selector parameters from the configuration
	 */
	public String getSelectorPrefix() {
		if (this.selectorPrefix == null) {
			try {
				this.selectorPrefix =
						Config.getInstance().getProperty(SELECTOR_PREFIX_PROPERTY,
								SELECTOR_PREFIX_DEFAULT);
			} catch (Exception e) {
				this.selectorPrefix = SELECTOR_PREFIX_DEFAULT;
			}
		}

		return this.selectorPrefix;
	}
	
	
	

	/**
	 * @param prefix
	 * @return
	 */
	public String getUniqueParameterName(String prefix) {
		String id = prefix + Integer.toHexString((int) (Math.random() * 10000));

		while (this.parameterNames.get(id) != null) {
			id = prefix + Integer.toHexString((int) (Math.random() * 10000));
		}

		return id;
	}
	
	
	
			
	/**
	 * Provide phase information e.g. signal that you're currently in submit-phase.
	 * @param chibaBean : The chiba processor
	 * @throws XFormsException
	 */
	private void addPhaseListener(ChibaBean chibaBean)
			throws XFormsException {
		
		
		EventTarget eventTarget = (EventTarget) this.chibaBean.getXMLContainer().getDocumentElement();
		
		//For setting init phase
		eventTarget.addEventListener(EventFactory.READY, this, true);
		
		//For setting submit phase
		eventTarget.addEventListener(EventFactory.SUBMIT, this, true);
	}
	
	
	
	
	/**
	 * Checks whether we have multipart or urlencoded request and processes it accordingly. After updating
	 * the data, a reacalculate, revalidate refresh sequence is fired and the found trigger is executed.
	 *
	 * @param request Servlet request
	 * @throws XFormsException todo: implement action block behaviour
	 */
	public void handleRequest(HttpServletRequest request)
			throws XFormsException {
		
		//Trigger
		String trigger = null;
		trigger = processUrlencodedRequest(request, trigger);

		//Finally activate trigger if any
		if (trigger != null) {
			this.chibaBean.dispatch(trigger, EventFactory.DOM_ACTIVATE);
		}
	}
	
	
	
	/* TODO SEMPERE Benjamin - Check the JavaDoc*/
	/**
	 * @param request
	 * @param trigger
	 * @return
	 * @throws XFormsException
	 */
	private String processUrlencodedRequest(HttpServletRequest request,
											String trigger)
			throws XFormsException {
				
		//Iterate request parameters
		Enumeration names = request.getParameterNames();
		while (names.hasMoreElements()) {
			String name = names.nextElement().toString();
			String[] values = request.getParameterValues(name);

			// [1] handle data
			handleData(name, values);

			// [3] handle selector
			handleSelector(name, values[0]);

			// [2] handle trigger
			trigger = handleTrigger(trigger, name);
		}
		return trigger;
	}
	
	
	
	
	/**
	 * @param name
	 * @throws XFormsException
	 */
	private void handleData(String name, String[] values)
			throws XFormsException {
		if (name.startsWith(getDataPrefix())) {
			String id = (String) this.parameterNames.get(name);

			//Assemble new control value
			String newValue;

			if (values.length > 1) {
				StringBuffer buffer = new StringBuffer(values[0]);

				for (int i = 1; i < values.length; i++) {
					buffer.append(" ").append(values[i]);
				}

				newValue = buffer.toString().trim();
			} else {
				newValue = values[0];
			}

			//Control updtating
			this.chibaBean.updateControlValue(id, newValue);
		}
	}
	
	
	
	
	/**
	 * @param name
	 * @throws XFormsException
	 */
	private void handleSelector(String name, String value)
			throws XFormsException {
		if (name.startsWith(getSelectorPrefix())) {
			String id = name.substring(getSelectorPrefix().length());
			int index = Integer.valueOf(value).intValue();
			Repeat repeat = (Repeat) this.chibaBean.getContainer().lookup(id);

			// set repeat index
			repeat.setIndex(index);
		}
	}




	/**
	 * @param trigger
	 * @param name
	 * @return
	 */
	private String handleTrigger(String trigger, String name) {
		if ((trigger == null) && name.startsWith(getTriggerPrefix())) {
			String parameter = name;
			int x = parameter.lastIndexOf(".x");
			int y = parameter.lastIndexOf(".y");

			if (x > -1) {
				parameter = parameter.substring(0, x);
			}

			if (y > -1) {
				parameter = parameter.substring(0, y);
			}

			// keep trigger id
			trigger = (String) this.parameterNames.get(parameter);
		}
		return trigger;
	}
	
	
	
	/**
	 * 
	 * @param key
	 * @return
	 */
	public Object getParameter(String key) {
		return this.webContext.getProperty((String) key);
	}



	/**
	 * 
	 * @param key
	 * @param val
	 * @return
	 */
	public Object setParameter(String key, Object val) {
		this.webContext.setProperty((String) key, val);
		return val;
	}
}
