<?xml version="1.0" encoding="UTF-8"?>

<!--
ref2bind.xsl

Creates xforms:bind attributes  and their appropriate bind elements 
from xforms:ref or xforms:nodeset attributes found on controls.

The input document mustn't contain any bind elements otherwise
processing is stopped without modifications.


@Author Mark Dimon ( mark@markdimon.com )
@Author Joern Turner 
$Ver: $
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
							  xmlns:xforms="http://www.w3.org/2002/xforms"
							  xmlns:chiba="http://chiba.sourceforge.net/2003/08/xforms"
							  exclude-result-prefixes="chiba xsl xsd xforms ev xalan" 
							  xmlns:xalan="http://xml.apache.org/xalan">
	<xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>

<xsl:variable name="binds-exist" select="boolean(//xforms:model/xforms:bind)"/>

<xsl:variable name="first_model" select="//xforms:model[position()=1]"/>
<xsl:variable name="first_model_id" select="$first_model/@xforms:id"/>
<xsl:variable name="default_model_id" select="$first_model_id"/>
<xsl:variable name="has_empty_id_model" select="//xforms:model[ string(@xforms:id)=string('')]"/>

<xsl:template match="/">
	<xsl:if test='$binds-exist'>
		<xsl:message terminate='yes'>Binds already exist. Exiting...</xsl:message>
	</xsl:if>
	
	<xsl:copy>
		<xsl:apply-templates />
	</xsl:copy>
	
	
</xsl:template>

<xsl:template match="*|@*|comment()|text()">
	<xsl:copy>
		<xsl:apply-templates select="*|@*|comment()|text()"/>
	</xsl:copy>
</xsl:template>

<xsl:template match="*|@*|comment()|text()" mode="model">
	<xsl:copy>
		<xsl:apply-templates select="*|@*|comment()|text()" mode="model"/>
	</xsl:copy>
</xsl:template>

<xsl:template name="copy-attributes">
	<xsl:for-each select="@*">
		<xsl:copy/>
	</xsl:for-each>
</xsl:template>

<xsl:template match="xforms:model" priority="5">
	
	<xsl:message>MODEL-ID:<xsl:value-of select='@xforms:id'/></xsl:message>
	<xsl:copy>
		<xsl:call-template name="copy-attributes"/>
		
		<!-- copy instance, submission, schema ... -->
		<xsl:apply-templates mode="model"/>
		
		<xsl:message>
                constructing binding-information - overwriting existing bindings
        </xsl:message>
		<xsl:call-template name="create-binds-from-ui">
			<xsl:with-param name="model_id" select="./@xforms:id"/>
		</xsl:call-template>
		
	</xsl:copy>
</xsl:template>

<!--
<xsl:template match="xforms:bind" mode="model">
	<xsl:copy>
		<xsl:apply-templates select="@*"/>


		<xsl:attribute name="xforms:ref">
			<xsl:call-template name="absRef"/>
		</xsl:attribute>


		<xsl:choose>
			<xsl:when test="@xforms:id">
			</xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="xforms:id">
					<xsl:value-of select="generate-id()"/>
				</xsl:attribute>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:copy>
	<xsl:apply-templates select="xforms:bind" mode="model"/>
</xsl:template>
-->

<!-- writes binds on UI elements  -->
<xsl:template match="xforms:*">
<xsl:message>xforms element</xsl:message>
	<xsl:copy>
		<xsl:apply-templates select="@*"/>
		<xsl:if test='@xforms:ref or @xforms:nodeset'>
			<xsl:call-template name="generate-bind-id" />
		</xsl:if>
		
		<!--
		<xsl:choose>
			<xsl:when test="@xforms:ref">
				<xsl:attribute name="absRef">
					<xsl:call-template name="absRef"/>
				</xsl:attribute>
				<xsl:call-template name="generate-bind-id" />
			</xsl:when>
			<xsl:when test="@xforms:nodeset">
				<xsl:attribute name="absRef">
					<xsl:call-template name="absRef"/>
				</xsl:attribute>
				<xsl:call-template name="generate-bind-id" />
			</xsl:when>
		</xsl:choose>
		-->
		
		<xsl:apply-templates select="*|comment()|text()"/>
	</xsl:copy>
</xsl:template>

<xsl:template match="node()" mode="copy">
	<xsl:variable name="elem" select="name()"/>
	<xsl:choose>
		<xsl:when test="@xforms:ref or @xforms:nodeset">
			<xsl:variable name="ref-id" select="generate-id()"/>
			<xsl:element name="{$elem}">
				<xsl:attribute name="xforms:bind">
					<xsl:value-of select="$ref-id"/>
				</xsl:attribute>
				<xsl:apply-templates select="@*[not(@xforms:bind)]" mode="copy"/>
				<xsl:apply-templates mode="copy"/>
			</xsl:element>
		</xsl:when>
		<xsl:otherwise>
			<xsl:copy>
				<xsl:apply-templates select="@*" mode="copy"/>
				<xsl:apply-templates mode="copy"/>
			</xsl:copy>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="create-binds-from-ui">
	<xsl:param name="model_id"/>

	<xsl:variable name="default_model" select="$first_model_id"/>
    
	<xsl:apply-templates select="/*" mode="create">
		<xsl:with-param name="model_id" select="$model_id"/>
		<xsl:with-param name="default_model" select="$default_model"/>
	</xsl:apply-templates>
	<xsl:comment>end of ui generated binds</xsl:comment>
</xsl:template>

<xsl:template match="*" mode="create">
	<xsl:param name="model_id"/>
	<xsl:param name="default_model"/>

	<xsl:apply-templates select="*" mode="create">
		<xsl:with-param name="model_id" select="$model_id"/>
		<xsl:with-param name="default_model" select="$default_model"/>
	</xsl:apply-templates>
</xsl:template>

<xsl:template match="xforms:*" mode="create">
	<xsl:param name="model_id"/>
	<xsl:param name="default_model"/>
	
	<xsl:message>
	create binding Attribute for:<xsl:value-of select='name()'/>
	</xsl:message>

	<xsl:variable name="id">
		<xsl:choose>
			<xsl:when test="string(@xforms:model)=string('')">
				<xsl:value-of select="$default_model"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="@xforms:model"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

<xsl:choose>
	<xsl:when test='string($id)=string($model_id)'>
		<xsl:if test="@xforms:ref or @xforms:nodeset">
	<!--
	                    			<xsl:if test=" not ( name()='xforms:setValue'  ||
	                             name()='xforms:loadURI' || 
	                             name()='xforms:insert'  ||
	                             name()='xforms:delete'  ||
	                             name()='xforms:message' ) ">
	                    -->
	
				<xsl:element name="xforms:bind">
				
					<xsl:attribute name="xforms:id">
						<xsl:value-of select="generate-id()"/>
					</xsl:attribute>
					<xsl:attribute name="xforms:nodeset">
						<xsl:call-template name="binding-attribute"/>
					</xsl:attribute>
	<!-- default datatype is string -->
	
					<xsl:attribute name="xforms:type">
						<xsl:text>string</xsl:text>
					</xsl:attribute>
	
					<xsl:apply-templates select="*" mode="create">
						<xsl:with-param name="model_id" select="$model_id"/>
						<xsl:with-param name="default_model" select="$default_model"/>
					</xsl:apply-templates>
					
				</xsl:element>
		</xsl:if>
	</xsl:when>
	<xsl:otherwise>
		<xsl:apply-templates select="*" mode="create">
						<xsl:with-param name="model_id" select="$model_id"/>
						<xsl:with-param name="default_model" select="$default_model"/>
		</xsl:apply-templates>
	</xsl:otherwise>
</xsl:choose>
<!--
	<xsl:if test="string($id)=string($model_id)">
		<xsl:if test="@xforms:ref or @xforms:nodeset">

			<xsl:element name="xforms:bind">
			
				<xsl:attribute name="xforms:id">
					<xsl:value-of select="generate-id(..)"/>
				</xsl:attribute>
				<xsl:attribute name="xforms:nodeset">
					<xsl:call-template name="binding-attribute"/>
				</xsl:attribute>

				<xsl:attribute name="xforms:type">
					<xsl:text>string</xsl:text>
				</xsl:attribute>

				<xsl:apply-templates select="*" mode="create">
					<xsl:with-param name="model_id" select="$model_id"/>
					<xsl:with-param name="default_model" select="$default_model"/>
				</xsl:apply-templates>
				
			</xsl:element>
		</xsl:if>
	</xsl:if>
	-->
</xsl:template>

<xsl:template name='binding-attribute'>
	<xsl:choose>
		<xsl:when test='@xforms:ref'>
			<xsl:value-of select='@xforms:ref'/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select='@xforms:nodeset'/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<xsl:template name="generate-bind-id">
	<xsl:if test="not(@xforms:bind)">
		<xsl:attribute name="xforms:bind">
			<xsl:value-of select="generate-id()"/>
		</xsl:attribute>
	</xsl:if>
</xsl:template>

<!--
if any actions are not wrapped by <xforms:action>, it is done here
-->
<xsl:template name="action">
	<xsl:choose>
		<xsl:when test="name(..)='xforms:action'">
			<xsl:copy>
				<xsl:apply-templates select="@*"/>
			</xsl:copy>
		</xsl:when>
		<xsl:otherwise>
			<xforms:action>
				<xsl:copy>
					<xsl:apply-templates select="@*"/>
				</xsl:copy>
			</xforms:action>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<xsl:template match="xforms:dispatch">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:insert">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:delete">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:setValue">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:setFocus">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:setRepeatCursor">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:revalidate">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:recalculate">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:refresh">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:submitInstance">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:resetInstance">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:message">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:script">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:loadURI">
	<xsl:call-template name="action"/>
</xsl:template>
<xsl:template match="xforms:toggle">
	<xsl:call-template name="action"/>
</xsl:template>
</xsl:stylesheet>
<!--
$Log: ref2bind.xsl,v $
Revision 1.1  2004/05/13 07:41:50  sempereb
Formgenerator-1.0 application

Revision 1.2  2003/08/05 08:57:23  joernt
xforms + chiba namespaces adapted

Revision 1.1.1.1  2003/05/23 14:54:09  unl
no message

Revision 1.1  2002/10/31 23:32:23  joernt
converts ref attributes to bind elements

Revision 1.2  2002/09/17 00:42:04  joernt
corrected XForms namespace

Revision 1.1  2002/09/11 23:10:16  joernt
two new stylesheets:
- index.xsl indexed all xforms elements with an ID
- ref2bind.xsl generates bind elements for documents which haven't any

Revision 1.2  2002/03/05 01:35:49  mdimon
Now Instance in own document , it is now assumed instance data is single rooted when passed in Instance.java. Improvments to Xpath extensions , Some way on handling models correctly.

Revision 1.1  2002/02/28 04:31:24  mdimon
pre-transform to dom on loading a xform

-->
