package hero.client.test;

/*
*
* NodeTests.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import javax.security.auth.login.LoginContext;

import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHome;
import hero.interfaces.BnUserHome;
import hero.interfaces.BnUserPropertyValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;
import hero.util.HeroException;

import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;

import hero.interfaces.UserRegistration;
import hero.interfaces.UserRegistrationHome;
import hero.interfaces.UserRegistrationUtil;

public class UserSessionTests extends TestCase {

    private BnProjectHome projecth;
    private ProjectSessionHome projectsesh;
    private BnUserHome userh;
    private UserSessionHome usersessionh;

    public UserSessionTests(String testname) {
	super(testname);
    }
    
    public static TestSuite suite() {
	return new TestSuite(UserSessionTests.class);
    }

    public void setUp() throws Exception {
    }

   public void testGetProjectListSession() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	ProjectSession p = projectsesh.create();
	p.initProject("Failure");
	p.addNode("t1",1);
	p.addNode("t2",1);
	p.addEdge("t1","t2");
	usersessionh=UserSessionUtil.getHome();
	UserSession usersession = usersessionh.create();
	ProjectSession lproj1 = projectsesh.create();
	lproj1.initProject("p1SS");
	ProjectSession lproj2 = projectsesh.create();
	lproj2.initProject("p2SS");		
	System.out.println("usersession home ?"+usersessionh);

	System.out.println("usersession ?"+usersession);
	Collection c = usersession.getProjectList();
	assertTrue("getProjectList Error",c.size() != 0);
	Collection c2 = usersession.getToDoList("p1SS");
	Collection c3 = usersession.getToDoList("p2SS");
	usersession.startActivity("Failure","t1");
	usersession.startActivity("Failure","t2");
	usersession.terminateActivity("Failure","t1");
    }

    public void testGetProjectList() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	UserSession usersession = usersessionh.create();
	ProjectSession lproj1 = projectsesh.create();
	lproj1.initProject("p1");
	ProjectSession lproj2 = projectsesh.create();
	lproj2.initProject("p2");		
	System.out.println("usersession home ?"+usersessionh);

	System.out.println("usersession ?"+usersession);
	Collection c = usersession.getProjectList();
	assertTrue("getProjectList Error",c.size() != 0);
    }

    
    // getToDoList test method in order to verify user ready activities
    

    public void testGetToDoList1() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("p3");		
	
	proj.addNode("name1",Constants.Nd.AND_JOIN_NODE);
	proj.setUserRole("admin","InitialRole");

	UserSession usersession = usersessionh.create();
	Collection c = usersession.getToDoList("p3");
	Iterator i = c.iterator();
	assertTrue("getToDoList Error",((String)i.next()).equals("name1"));
    }

    public void testGetToDoList2() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("p4");		
	
	proj.addNode("name1",Constants.Nd.AND_JOIN_NODE);
	proj.addNode("name2",Constants.Nd.AND_JOIN_NODE);
	
	proj.setUserRole("admin","InitialRole");

	UserSession usersession = usersessionh.create();
	Collection c = usersession.getToDoList("p4");
	Iterator i = c.iterator();
	String node1 = (String)i.next();
	String node2 = (String)i.next();

	assertTrue("getToDoList Error",node1.equals("name1") || node1.equals("name2"));
	assertTrue("getToDoList Error",node2.equals("name1") || node2.equals("name2"));
    }

   
    // getActivityList test method in order to verify startActivity method
    // and obtain new executing users activities
    

    public void testGetActivityList() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("p5");		
	
	proj.addNode("NodeExecuting",Constants.Nd.AND_JOIN_NODE);
	
	proj.setUserRole("admin","InitialRole");
	 
	UserSession usersession = usersessionh.create();
	Collection c=usersession.getActivityList("p5");
	assertTrue("getActivityList Error",c.size()==0);
	usersession.startActivity("p5","NodeExecuting");
	c = usersession.getActivityList("p5");
	Iterator i = c.iterator();
	assertTrue("getActivityList Error",((String)i.next()).equals("NodeExecuting"));
    }
    
    public void testGetNode() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	usersessionh=UserSessionUtil.getHome();
	ProjectSession proj = projectsesh.create();
	proj.initProject("testgetnode");		
	proj.addNode("pouet",Constants.Nd.AND_JOIN_NODE);
	UserSession usr=usersessionh.create();
	try {
	    BnNodeValue nv=usr.getNode("testgetnode","pouet");
	} catch (HeroException h) {
	    assertTrue("Should be a success",false);
	}
    }
	
    public void testProperty() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();

	usr.setUserProperty("prop1","val1");
	usr.setUserProperty("prop2","val2");
	usr.setUserProperty("prop3","val3");
	
    }	
    public void testDeleteProperty() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	usr.deleteUserProperty("prop2");	
	usr.deleteUserProperty("prop3");	
    }	

    public void testGetProperties() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	
	Collection c=usr.getUserProperties();
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnUserPropertyValue p=(BnUserPropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    assertTrue("Get Properties",key.equalsIgnoreCase("prop1"));
	    
	}
    }


  public void testSpeedCheckOneCallRMI() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	usr.getUser();
	usr.getUserPassword();	
	
  }
  public void testSpeedCheckTenCallRMI() throws Exception {
	usersessionh=UserSessionUtil.getHome();
	UserSession usr=usersessionh.create();
	for(int i=0;i<10;i++){
	    usr.getUser();
	    usr.getUserPassword();	
	}
  }


  public void testSpeedCheckOneCallSOAP() throws Exception {
         //String endpoint = "http://localhost:8080/jboss-net/services/UserSession";
	 String endpoint =  System.getProperty("bonita.host")+"/bonita_ws/services/UserSession";
     
	 Service  service = new Service();
	 Call     call    = (Call) service.createCall();
	 
	 call.setTargetEndpointAddress( new java.net.URL(endpoint) );

	 call.setMaintainSession(true);
	 call.setUsername("admin");
	 call.setPassword("toto");

	 call.setOperationName("getUser");
	 String resp = (String) call.invoke( new Object[] {} );	           
	 call.setOperationName("getUserPassword");	 
	 String ret2 = (String) call.invoke( new Object[] {} );	 
	 
  }

 public void testSpeedCheckTenCallSOAP() throws Exception {
    //     String endpoint = "http://localhost:8080/jboss-net/services/UserSession";
	String endpoint =  System.getProperty("bonita.host")+"/bonita_ws/services/UserSession";
     
	 Service  service = new Service();
	 Call     call    = (Call) service.createCall();
	 
	 call.setTargetEndpointAddress( new java.net.URL(endpoint) );

	 call.setMaintainSession(true);
	 call.setUsername("admin");
	 call.setPassword("toto");

	 for(int i=0;i<10;i++){
	     call.setOperationName("getUser");
	     String resp = (String) call.invoke( new Object[] {} );	           
	     call.setOperationName("getUserPassword");	 
	     String ret2 = (String) call.invoke( new Object[] {} );
	 } 
  }
public void testGetProjectInstances() throws Exception {
	projectsesh = ProjectSessionUtil.getHome();
	ProjectSession p = projectsesh.create();
	p.initProject("GPI");
	p.initProject("GPIGPI");
	String name1 = p.instantiateProject("GPI");
	String name2 = p.instantiateProject("GPI");

	usersessionh = UserSessionUtil.getHome();
	UserSession usr = usersessionh.create();

	Collection instances = usr.getProjectInstancesNames("GPI");
	assertTrue(
		"getProjectInstances Error",
		instances.contains("GPI_instance1"));
	assertTrue(
		"getProjectInstances Error",
		instances.contains("GPI_instance2"));
}

public void testInstancesByProperty() throws Exception {
	ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
	ProjectSession ps = projectSessionh.create();
	ps.initProject("instProp");

	ProjectSession ps2 = projectSessionh.create();
	ps2.initProject("instProp2");

	ProjectSession ps3 = projectSessionh.create();
	ps3.initProject("instProp3");

	ps.setProperty("id", "1234");
	ps2.setProperty("id", "1234");
	ps3.setProperty("id2", "1234");

	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession usersession = uHome.create();
	Collection names = usersession.getInstancesByPropertyNames("id", "1234");

	assertTrue(
		"Error in Instances by Property. Project error",
		names.size() == 0);
	String prop1 = ps.instantiateProject("instProp");
	String prop2 = ps2.instantiateProject("instProp2");
	String prop3 = ps3.instantiateProject("instProp3");
	ps.setProperty("id", "1234");
	ps2.setProperty("id", "1234");
	ps3.setProperty("id2", "1234");
	names = usersession.getInstancesByPropertyNames("id", "1234");
	assertTrue(
		"Error in Instances by Property. Instance error",
		names.size() == 2);
}

public void testProjectsByProperty() throws Exception {
	ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
	ProjectSession ps = projectSessionh.create();
	ps.initProject("projProp");

	ProjectSession ps2 = projectSessionh.create();
	ps2.initProject("projProp2");

	ps.setProperty("id", "123");
	ps2.setProperty("id", "123");

	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession usersession = uHome.create();
	Collection names = usersession.getProjectsByPropertyNames("id", "123");

	assertTrue("Error in Projects by Property", names.size() == 2);
	String prop1 = ps.instantiateProject("projProp");
	String prop2 = ps2.instantiateProject("projProp2");
	ps.setProperty("id", "123");
	ps2.setProperty("id", "123");
	names = usersession.getProjectsByPropertyNames("id", "123");
	assertTrue("Error in Projects by Property", names.size() == 2);
}

public void testGetInstancesList() throws Exception {
	UserRegistrationHome urHome = UserRegistrationUtil.getHome();
	UserRegistration userReg = urHome.create();
	userReg.userCreate(
		"GetInst",
		"GetInst",
		"miguel.valdes-faura@ext.bull.net");

	ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
	ProjectSession ps = projectSessionh.create();
	ps.initProject("GetInstancesList");
	ps.addNode("node1", 1);
	ps.addUser("GetInst");
	ps.setNodeRole("node1", "GetInst");

	ProjectSession ps2 = projectSessionh.create();
	ps2.initProject("GetInstancesList2");

	ProjectSession ps3 = projectSessionh.create();
	ps3.initProject("GetInstancesList3");
	ps3.addNode("node1", 1);
	ps3.addUser("GetInst");
	ps3.setNodeRole("node1", "GetInst");

	SimpleCallbackHandler handler =
		new SimpleCallbackHandler("GetInst", "GetInst".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();

	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession usersession = uHome.create();

	Collection names = usersession.getInstancesListNames();
	assertTrue(
		"Error in getInstancesList. GetInstancesListNames",
		names.size() == 0);

	names = usersession.getProjectListNames();
	assertTrue(
		"Error in getInstancesList. GetProjectListNames",
		names.size() == 2);

	names = usersession.getInstancesList();
	assertTrue(
		"Error in getInstancesList. GetInstancesList",
		names.size() == 0);

	names = usersession.getProjectList();
	assertTrue("Error in getInstancesList. GetProjectList", names.size() == 2);

	ps.instantiateProject("GetInstancesList");
	ps.instantiateProject("GetInstancesList3");

	names = usersession.getInstancesListNames();
	assertTrue(
		"Error in getInstancesList. GetInstancesListNames",
		names.size() == 2);

	names = usersession.getProjectListNames();
	assertTrue(
		"Error in getInstancesList. GetProjectListNames",
		names.size() == 4);

	names = usersession.getInstancesList();
	assertTrue(
		"Error in getInstancesList. GetInstancesList",
		names.size() == 2);

	names = usersession.getProjectList();
	assertTrue("Error in getInstancesList. GetProjectList", names.size() == 4);

	char[] password2 = { 't', 'o', 't', 'o' };
	handler = new SimpleCallbackHandler("admin", password2);
	lc = new LoginContext("TestClient", handler);
	lc.login();
}

public void testGetTodoListAllInstances() throws Exception {
	UserRegistrationHome urHome = UserRegistrationUtil.getHome();
	UserRegistration userReg = urHome.create();
	userReg.userCreate("tdai", "tdai", "miguel.valdes-faura@ext.bull.net");

	ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
	ProjectSession ps = projectSessionh.create();
	ps.initProject("TDAI");
	ps.addNode("node1", 1);
	ps.addNode("node2", 1);
	ps.addUser("tdai");
	ps.setNodeRole("node1", "tdai");

	ProjectSession ps2 = projectSessionh.create();
	ps2.initProject("TDAI2");
	ps2.addNode("node2", 1);

	ProjectSession ps3 = projectSessionh.create();
	ps3.initProject("TDAI3");
	ps3.addNode("node1", 1);
	ps3.addNode("node2", 1);
	ps3.addUser("tdai");
	ps3.setNodeRole("node1", "tdai");

	SimpleCallbackHandler handler =	new SimpleCallbackHandler("tdai", "tdai".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ps.instantiateProject("TDAI");
	ps2.instantiateProject("TDAI2");
	ps3.instantiateProject("TDAI3");

	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession usersession = uHome.create();

	Collection names = usersession.getToDoListAllInstances();
	assertTrue("Error in getTodoListAllInstances",names.size() == 2);

	char[] password2 = { 't', 'o', 't', 'o' };
	handler = new SimpleCallbackHandler("admin", password2);
	lc = new LoginContext("TestClient", handler);
	lc.login();
}

public void testGetActivityListAllInstances() throws Exception {
	UserRegistrationHome urHome = UserRegistrationUtil.getHome();
	UserRegistration userReg = urHome.create();
	userReg.userCreate("taai", "taai", "miguel.valdes-faura@ext.bull.net");

	ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
	ProjectSession ps = projectSessionh.create();
	ps.initProject("TAAI");
	ps.addNode("node1", 1);
	ps.addNode("node2", 1);
	ps.addUser("taai");
	ps.setNodeRole("node1", "taai");

	ProjectSession ps2 = projectSessionh.create();
	ps2.initProject("TAAI2");
	ps2.addNode("node2", 1);

	ProjectSession ps3 = projectSessionh.create();
	ps3.initProject("TAAI3");
	ps3.addNode("node1", 1);
	ps3.addNode("node2", 1);
	ps3.addUser("taai");
	ps3.setNodeRole("node1", "taai");

	SimpleCallbackHandler handler =	new SimpleCallbackHandler("taai", "taai".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();

	String name1 = ps.instantiateProject("TAAI");
	String name2 = ps2.instantiateProject("TAAI2");
	String name3 = ps3.instantiateProject("TAAI3");

	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession usersession = uHome.create();

	Collection names = usersession.getActivityListAllInstances();
	assertTrue("Error in getActivityAllInstancesList.1 ",names.size() == 0);
	
	usersession.startActivity(name1,"node1");
	
	names = usersession.getActivityListAllInstances();
	assertTrue("Error in getActivityAllInstancesList.2 ",names.size() == 1);
	
	usersession.startActivity(name3,"node1");
	
	names = usersession.getActivityListAllInstances();
	assertTrue("Error in getActivityAllInstancesList.3 ",names.size() == 2);
	
	usersession.terminateActivity(name3,"node1");

	names = usersession.getActivityListAllInstances();
	assertTrue("Error in getActivityAllInstancesList.4 ",names.size() == 1);

	char[] password2 = { 't', 'o', 't', 'o' };
	handler = new SimpleCallbackHandler("admin", password2);
	lc = new LoginContext("TestClient", handler);
	lc.login();
}

public void testGetTodoListByProperty() throws Exception {
	UserRegistrationHome urHome = UserRegistrationUtil.getHome();
	UserRegistration userReg = urHome.create();
	userReg.userCreate("tdp", "tdp", "miguel.valdes-faura@ext.bull.net");

	ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
	ProjectSession ps = projectSessionh.create();
	ps.initProject("TDP");
	ps.addNode("node1", 1);
	ps.setProperty("test","111");
	ps.setNodeProperty("node1","test","111");
	ps.setNodeProperty("node1","test","1112");
	ps.addNode("node2", 1);
	ps.addUser("tdp");
	ps.setNodeRole("node1", "tdp");
	ps.setNodeRole("node2", "tdp");

	ProjectSession ps2 = projectSessionh.create();
	ps2.initProject("TDP2");
	ps2.addNode("node2", 1);

	ProjectSession ps3 = projectSessionh.create();
	ps3.initProject("TDP3");
	ps3.addNode("node1", 1);
	ps3.setNodeProperty("node1","test","111");
	ps3.addNode("node2", 1);
	ps3.setNodeProperty("node2","test3","111");
	ps3.addUser("tdp");
	ps3.setNodeRole("node1", "tdp");

	SimpleCallbackHandler handler =	new SimpleCallbackHandler("tdp", "tdp".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();
	
	String name = ps.instantiateProject("TDP");
	ps.setNodeProperty("node2","test","111");
	ps.setProperty("test","111");
	ps2.instantiateProject("TDP2");
	ps3.instantiateProject("TDP3");

	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession usersession = uHome.create();

	Collection names = usersession.getToDoListByProperty("test","111");
	assertTrue("Error in getTodoListByProperty.1", names.size() == 2);

	ps.setNodeProperty("node1","test","111");
	ps.setNodeProperty("node2","test3","11122");
	
	ps.setProperty("test","111");
	ps.setProperty("test3","11122");

	names = usersession.getToDoListByProperty("test","111");
	assertTrue("Error in getTodoListByProperty.2", names.size() == 2);

	names = usersession.getToDoListByProperty("test","1111");
	assertTrue("Error in getTodoListByProperty.3", names.size() == 0);
	
	usersession.startActivity(name,"node1");

	names = usersession.getToDoListByProperty("test","111");
	assertTrue("Error in getTodoListByProperty.4", names.size() == 1);

	char[] password2 = { 't', 'o', 't', 'o' };
	handler = new SimpleCallbackHandler("admin", password2);
	lc = new LoginContext("TestClient", handler);
	lc.login();
}

public void testActivityListByProperty() throws Exception {
	UserRegistrationHome urHome = UserRegistrationUtil.getHome();
	UserRegistration userReg = urHome.create();
	userReg.userCreate("alp", "alp", "miguel.valdes-faura@ext.bull.net");

	ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
	ProjectSession ps = projectSessionh.create();
	ps.initProject("ALP");
	ps.addNode("node1", 1);
	ps.setProperty("test","111");
	ps.setNodeProperty("node1","test","111");
	ps.setNodeProperty("node1","test","1112");
	ps.addNode("node2", 1);
	ps.addUser("alp");
	ps.setNodeRole("node1", "alp");
	ps.setNodeRole("node2", "alp");

	ProjectSession ps2 = projectSessionh.create();
	ps2.initProject("ALP2");
	ps2.addNode("node2", 1);

	ProjectSession ps3 = projectSessionh.create();
	ps3.initProject("ALP3");
	ps3.addNode("node1", 1);
	ps3.setNodeProperty("node1","test","111");
	ps3.setProperty("test","111");
	ps3.setProperty("test","111");
	ps3.setNodeProperty("node1","test","111");
	ps3.addNode("node2", 1);
	ps3.setNodeProperty("node2","test3","111");
	ps3.addUser("alp");
	ps3.setNodeRole("node1", "alp");

	SimpleCallbackHandler handler =	new SimpleCallbackHandler("alp", "alp".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();
	
	String name = ps.instantiateProject("ALP");
	ps.setProperty("test","111");
	ps.setNodeProperty("node2","test","111");
	ps2.instantiateProject("ALP2");
	ps3.instantiateProject("ALP3");

	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession usersession = uHome.create();

	Collection names = usersession.getActivityListByProperty("test","111");
	assertTrue("Error in getActivityListByProperty.1", names.size() == 0);
	
	usersession.startActivity(name,"node2");

	names = usersession.getActivityListByProperty("test","111");
	assertTrue("Error in getActivityListByProperty.2", names.size() == 1);

	ps.setNodeProperty("node1","test","111");
	ps.setNodeProperty("node2","test3","11122");
	ps.setProperty("test","111");
	ps.setProperty("test3","11122");

	names = usersession.getActivityListByProperty("test","111");
	assertTrue("Error in getActivityListByProperty.2", names.size() == 1);

	names = usersession.getToDoListByProperty("test","1111");
	assertTrue("Error in getActivityListByProperty.3", names.size() == 0);
	
	usersession.terminateActivity(name,"node2");

	names = usersession.getActivityListByProperty("test","111");
	assertTrue("Error in getActivityListByProperty.4", names.size() == 0);

	char[] password2 = { 't', 'o', 't', 'o' };
	handler = new SimpleCallbackHandler("admin", password2);
	lc = new LoginContext("TestClient", handler);
	lc.login();
}
}
