/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnInstanceBean.java,v 1.3 Miguel Valdes Faura Exp $
 *
--------------------------------------------------------------------------
 */

package hero.entity;

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnInstanceLightValue;
import hero.interfaces.BnInstanceValue;
import hero.interfaces.BnInstanceLocal;
import hero.interfaces.BnInstanceLocalHome;
import hero.interfaces.InvalidValueException;
import hero.interfaces.BnInstancePK;

import hero.util.BonitaServiceLocator;
import hero.util.BonitaServiceException;

import java.util.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;

import org.apache.log4j.Category;

/**
 * The Entity bean represents a BnInstance
 *
 * @author Miguel Valdes Faura
 *
 * @ejb.bean name="BnInstance"
 *           display-name="BnInstance Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnInstance"
 *           local-jndi-name="ejb/hero/BnInstance_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"        
 *
 *
 * @ejb.ejb-ref ejb-name="hero/SequenceGenerator"
 *
 *
 * @ejb.permission unchecked="yes"
 * @ejb.pk generate="true" method-intf="remote"
 * 
 * @jonas.bean
 *     ejb-name="BnInstance"
 *     jndi-name="ejb/hero/BnInstance"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnInstance"
 * @jonas.cleanup create
 *
 * @ejb.finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(i) FROM BnInstance i"
 *     transaction-type="Supports"
 *
 * @ejb.finder signature="hero.interfaces.BnInstance findByName(java.lang.String pName )" result-type-mapping="Remote" 
 *     query="SELECT Object(i) From BnInstance as i Where i.name = ?1"
 *     transaction-type="Supports"
 * @ejb.finder signature="hero.interfaces.BnInstanceLocal findByName(java.lang.String pName )"  result-type-mapping="Local" 
 *     query="SELECT Object(i) From BnInstance as i Where i.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.finder signature="java.util.Collection findByUser(java.lang.String userName )" 
 *     query="SELECT OBJECT(i) From BnInstance as i, in (i.bnUsers) as u Where u.name = ?1"
 *     transaction-type="Supports"
 *
 * 
 * 
 * @ejb.finder signature="java.util.Collection findInstancesByProperty(java.lang.String key, java.lang.String value )" 
 *     query="SELECT OBJECT(i) From BnInstance as i, BnProjectProperty as prop
 * 					 Where prop.theKey = ?1
 * 					 AND prop.theValue = ?2
 * 					 AND p.id = prop.bnInstance.id"
 *     transaction-type="Supports"
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnInstance"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnInstanceLight"
 *
 * @ejb:transaction type="Supports"
 * @jboss.table-name table-name="BnInstance"
 *
 * @jboss.create-table create="true"
 *
 * @jboss.remove-table remove="false"
 *
 * @author $Author: valdesfm $
 **/
public abstract class BnInstanceBean implements EntityBean {

	// -------------------------------------------------------------------------
	// Members
	// -------------------------------------------------------------------------   // -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log =	Category.getInstance(BnInstanceBean.class);
	public EntityContext mContext;

	// -------------------------------------------------------------------------
	// Properties (Getters/Setters)
	// -------------------------------------------------------------------------  

	/**
	* Retrieve the BnInstance's id.
	*
	* @return Returns an int representing the id of this BnInstance.
	*
	* @ejb:pk-field
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="id"
	* @jonas.cmp-field-jdbc-mapping
	*    field-name="id"
	*    jdbc-field-name="id"
	* @ejb:transaction type="Supports"
	**/
	public abstract int getId();

	/**
	* Set the BnInstance's id.
	*
	* @param pId The id of this BnInstance. Is set at creation time.
	* @ejb:transaction type="Required"
	**/
	public abstract void setId(int pId);
	
	/**
	* Retrieve the BnInstance's Name.
	*
	* @return Returns an int representing the Name of this BnInstance.
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="name"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getName();

	/**
	* Set the BnInstance's Name.
	*
	* @param pName The Name of this BnInstance.  Is set at creation time.
	* @ejb:transaction type="Required"
	**/
	public abstract void setName(String pName);
	
	/**
	* Retrieve the BnInstance Creator.
	*
	* @return Returns a string representing the name of the creator.
	*
	* @ejb.interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="creator"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getCreator();

	/**
	* Set the Creator Name.
	*
	* @param pCreator The Creator of this Instance
	* @ejb:transaction type="Required"
	**/
	public abstract void setCreator(String pCreator);


	/**
	* Get parent of this instance (or the name of the instance if is not a child)
	*
	* @return Returns a String representing the parent name
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="parent"
	* @ejb:transaction type="Supports"
	**/

	public abstract String getParent();

	/**
	* Set the parent of this instance (or the name of the instance if is not a child)
	*
	* @param parent of this instance. Is set at creation time.
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public abstract void setParent(String parent);

	/**
	* @return Returns the State of this BnInstance
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="state"
	* @ejb:transaction type="Supports"
	**/
	public abstract int getState();

	/**
	* Specify the State of this BnInstance
	*
	* @param pState State of this BnInstance
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public abstract void setState(int pState);
	
	/**
	 * @return  the project of the instance
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="Instances" role-name="is-instance-of" cascade-delete="yes"
	 * @jboss:auto-key-fields 
	 * @jonas:automatic-pk="true"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract BnProjectLocal getBnProject();

    /**
    * Set the BnProject.
    * @ejb:interface-method view-type="local"
    * @param pProject The Name of the project.  Is set at creation time.
    * @ejb:transaction type="Required"
    **/
	public abstract void setBnProject(BnProjectLocal pProject);
	
	/**
	 * @return Return the nodes of the instance
	 * 
	 * @ejb.interface-method view-type="local"
	 * @ejb.relation name="InstanceNodes" role-name="instances-has-nodes"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnNodeValue"
	 *       compose-name="BnNode"
	 *       members="hero.interfaces.BnNodeLocal"
	 *       members-name="BnNode"
	 * @ejb:transaction type="Supports"
	 **/
	public abstract Collection getBnNodes();

	/**
	* Specify the Nodes of this BnInstance
	*
	* @param pNodes Nodes of this BnInstance
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnNodes(Collection pNodes);
	/**
	 * @return Return the users of the instance
	 * 
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="InstanceUsers" role-name="instance-has-users" target-multiple="yes"
	 * @jboss:auto-key-fields 
	 * @jonas :automatic-pk="true"
	 * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnUserLightValue"
	 *       aggregate-name="BnUser"
	 *       members="hero.interfaces.BnUserLocal"
	 *       members-name="BnUser"
	 * @ejb:transaction type="Supports"     
	 **/
	public abstract Collection getBnUsers();

	/**
	* Specify the Users of this BnInstance
	*
	* @param pUsers Users of this BnInstance
	*
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnUsers(Collection pUsers);

	/**
	 * @return Return the roles of the instance
	 * 
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="InstanceRoles" role-name="instance-has-roles" target-multiple="yes"
	 * @jboss:auto-key-fields 
	 * @jonas :automatic-pk="true"
	 * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnRoleValue"
	 *       aggregate-name="BnRole"
	 *       members="hero.interfaces.BnRoleLocal"
	 *       members-name="BnRole"     
	 * @ejb:transaction type="Supports"
	 **/
	public abstract Collection getBnRoles();

	/**
	* Specify the Roles of this BnInstance
	*
	* @param pRoles Roles of this BnInstance
	*
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnRoles(Collection pRoles);

	/**
	 * @return Return the creation date of the BnInstance
	 * @ejb:interface-method view-type="both"
	 * @ejb:persistence column-name="creationdate"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Date getCreationDate();

	/**
	 * @param pDate the creation date of the BnInstance
	 * @ejb:interface-method view-type="both"
	 * @ejb.value-object
	 *      match="light"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public abstract void setCreationDate(Date pDate);

	/**
	 * @return Return the modification date of the BnInstance
	 * @ejb:interface-method view-type="both"
	 * @ejb:persistence column-name="modificationdate"
	 * @ejb:transaction type="Supports"
	 * @ejb.value-object
	 *      match="light"
	 *
	 **/
	public abstract Date getModificationDate();

	/**
	 * @param pDate the creation date of the BnInstance
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public abstract void setModificationDate(Date pDate);

	/**
	 * @return  the properties of the instance
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="InstanceProjectprp" role-name="is-property-of"
	 * @jboss:auto-key-fields 
	 * 
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnProjectPropertyValue"
	 *       compose-name="BnPropertie"
	 *       members="hero.interfaces.BnProjectPropertyLocal"
	 *       members-name="BnPropertie"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Collection getBnProperties();

	/**
	* Set the properties of the instance
	* @ejb:interface-method view-type="both"
	* @param pPrp the properties of the instance
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnProperties(Collection pPrp);
		
	/**
	 * @returns the light value of the bean (without relationships)
	 * @ejb.interface-method view-type=both
	 */
	public abstract BnInstanceLightValue getBnInstanceLightValue();
	
	/**
	 * @returns the value of the bean
	 * @ejb.interface-method view-type=both
	 */
	public abstract BnInstanceValue getBnInstanceValue();

	/**
	 * @ejb.interface-method 
	 * @ejb:transaction type="Required"
	 */
	public abstract void setBnInstanceValue(BnInstanceValue v);

	/**
	* Retrieve the BnInstance's java tree
	*
	* @return Returns an ProjectValue representing the java tree
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="javaTree"
	* @ejb:transaction type="Supports"
	**/
	public abstract BnProjectValue getJavaTree();


	/**
	 * @param jt The java tree of this Instance
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 */
	public abstract void setJavaTree(BnProjectValue jt);


	// -------------------------------------------------------------------------
	// Framework Callbacks
	// -------------------------------------------------------------------------  

	/**
	* Create a BnInstance based on the supplied BnInstance Value Object.
	*
	*
	* @throws InvalidValueException If one of the values are not correct,
	*                               this will not roll back the transaction
	*                               because the caller has the chance to
	*                               fix the problem and try again
	* @throws EJBException If no new unique ID could be retrieved this will
	*                      rollback the transaction because there is no
	*                      hope to try again
	* @throws CreateException Because we have to do so (EJB spec.)
	*
	* @ejb:create-method view-type="both"
	**/
	public BnInstancePK ejbCreate(BnInstanceValue iv )
	   throws
		  InvalidValueException,
		  EJBException,
		  CreateException
	{
		setBnInstanceValue(iv);
		try {
			BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
		    this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.INSTANCE));
    	} catch (BonitaServiceException e) {
		   throw new CreateException(e.getMessage());
		}
	   // This is only possible in CMPs. Otherwise return a valid PK.
	   return null;
	}



	/**
	* Create a BnInstance based on the supplied BnInstance Value Object.
	*
	* @throws InvalidValueException If one of the values are not correct,
	*                               this will not roll back the transaction
	*                               because the caller has the chance to
	*                               fix the problem and try again
	* @throws EJBException If no new unique ID could be retrieved this will
	*                      rollback the transaction because there is no
	*                      hope to try again
	* @throws CreateException Because we have to do so (EJB spec.)
	*
	* @ejb:create-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public BnInstancePK ejbCreate()
		throws InvalidValueException, EJBException, CreateException {
		try {
			BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
			this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.INSTANCE));
		} catch (BonitaServiceException e) {
		   throw new CreateException(e.getMessage());
		}

		// This is only possible in CMPs. Otherwise return a valid PK.
		return null;
	}

	/**
	* Create a BnInstance based on project name and creator.
	*
	* @param pName the name of the instance
	* @param pCreator the creator of the instance
	*
	* @throws EJBException If no new unique ID could be retrieved this will
	*                      rollback the transaction because there is no
	*                      hope to try again
	* @throws CreateException Because we have to do so (EJB spec.)
	*
	* @ejb:create-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public BnInstancePK ejbCreate(String pName, String creator)
		throws InvalidValueException, EJBException, CreateException {
		log.debug("ejbCreate: pName=" + pName + " creator=" + creator);
		BnInstanceLocalHome instancehome;
		BnInstanceLocal instance;

		try {
			instancehome = hero.interfaces.BnInstanceUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new CreateException(be.getMessage());
		}
		try {
			instance = instancehome.findByName(pName);
			throw new CreateException("BnInstance " + pName + " already exist");
		} catch (FinderException fe) {

			try {
				BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
				this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.INSTANCE));
			} catch (BonitaServiceException e) {
				   throw new CreateException(e.getMessage());
			}
			this.setName(pName);
			this.setCreator(creator);
		}

		// This is only possible in CMPs. Otherwise return a valid PK.
		return null;
	}

	public void setEntityContext(EntityContext lContext) {
		mContext = lContext;
	}

	public void unsetEntityContext() {
		mContext = null;
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

	public void ejbLoad() {
	}

	public void ejbStore() {
	}

	public void ejbRemove() throws RemoveException {
	}
}
