/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: Timer.java,v 1.3 Miguel Valdes Faura Exp $
 *
--------------------------------------------------------------------------
 */


package hero.util;

import javax.management.*;
import javax.naming.Context;
import org.objectweb.jonas.service.ServiceException;

//import objectweb
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceManager;

public class Timer extends AbsServiceImpl implements hero.util.TimerMBean, NotificationBroadcaster {
    
	private MBeanServer mbeanServer = null;
	private MBeanServer server;
	private final static String SERVICE_NAME = "jonas:type=service,name=Timer";
	private ObjectName timerName;
	javax.management.timer.Timer timer;

	public void doInit(Context ctx) throws ServiceException {

		// Get the JMX Server via JMX Service
		try {
			mbeanServer = 
				((JmxService) ServiceManager.getJmxService()).getJmxServer();
		} catch (ServiceException e) {
			// the JMX service may not be started
			mbeanServer = null;
		}
	}

	public void doStart() throws ServiceException {
		try {
			// Register MailService MBean : MailServiceImplMBean
			if (mbeanServer != null) {
				timer = new javax.management.timer.Timer();
			    mbeanServer.registerMBean(timer,new ObjectName(SERVICE_NAME));
				server = MBeanServerFactory.createMBeanServer();
				timerName = new ObjectName("jonas:type=service,name=Timer");
				server.createMBean("javax.management.timer.Timer", timerName,new Object[0], new String[0]);
				server.invoke(timerName, "start", new Object[0], new String[0]);				 
				timer.start();
			}


		} catch (MBeanException me) {
			throw new ServiceException("Cannot start the Timer Service (MBean Exception)" , me);
		} catch (ReflectionException re) {
			throw new ServiceException("Cannot start the Timer Service  (Reflexion Exception)" , re);
		} catch (InstanceNotFoundException infe) {
			throw new ServiceException("Cannot start the Timer Service (MBean Not Found)" , infe);
		} catch (InstanceAlreadyExistsException iae) {
			throw new ServiceException("Cannot start the Timer Service (Already Exists)" , iae);
		} catch (NotCompliantMBeanException ncmbe) {
			throw new ServiceException("Cannot start the Timer Service (MBean Not compliant error)" , ncmbe);
		} catch (MalformedObjectNameException mone) {
			throw new ServiceException("Cannot start the Timer Service (ObjectName Malformed)" , mone);
		}
		System.out.println("Timer :: Start Service");
	}

	public void doStop() throws ServiceException {

		if (mbeanServer != null) {
			try {
				mbeanServer.unregisterMBean(new ObjectName(SERVICE_NAME));
				server.invoke(timerName, "removeAllNotifications", new Object[0], new String[0]);
				server.invoke(timerName, "stop", new Object[0], new String[0]);
				server.unregisterMBean(timerName);
				MBeanServerFactory.releaseMBeanServer(server);
			} catch (InstanceNotFoundException infe) {
				throw new ServiceException("Cannot stop the Timer Service (MBean Not Found).", infe);
			} catch (Exception e) {
				throw new ServiceException("Cannot stop the Timer Service (JMX).", e);
			}
		}
		System.out.println("Timer :: Stop Service");
	}
	
}
