/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: TimerReceiver.java,v 1.3 Miguel Valdes Faura Exp $
 *
--------------------------------------------------------------------------
 */

package hero.util;
import javax.management.*;
import javax.naming.Context;
import org.objectweb.jonas.service.ServiceException;

//import objectweb
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceManager;

public class TimerReceiver extends AbsServiceImpl implements TimerReceiverMBean, NotificationListener {

	private MBeanServer mbeanServer = null;
	public final static String SERVICE_NAME = "jonas:type=service,name=TimerReceiver";

	public void doInit(Context ctx) throws ServiceException {
		// Get the JMX Server via JMX Service
		try {
			mbeanServer = 
				((JmxService) ServiceManager.getJmxService()).getJmxServer();
		} catch (ServiceException e) {
			// the JMX service may not be started
			mbeanServer = null;
		}
	}

	public void doStart() throws ServiceException {
		try {
			// Register MailService MBean : MailServiceImplMBean
			if (mbeanServer != null) {
				mbeanServer.registerMBean(this,new ObjectName(SERVICE_NAME));
			}
		} catch (InstanceAlreadyExistsException iae) {
			throw new ServiceException("Cannot start the TimerReceiver Service (Already Exists)" , iae);
		} catch (MBeanRegistrationException mbre) {
			throw new ServiceException("Cannot start the TimerReceiver Service (MBean registration error)" , mbre);
		} catch (NotCompliantMBeanException ncmbe) {
			throw new ServiceException("Cannot start the TimerReceiver Service (MBean Not compliant error)" , ncmbe);
		} catch (MalformedObjectNameException mone) {
			throw new ServiceException("Cannot start the TimerReceiver Service (ObjectName Malformed)" , mone);
		}
		System.out.println("TimerReceiver :: Start Service");
	}


	public void handleNotification(Notification n, Object handback) {
	/*try{
		JMSServicesSessionLocal jms;
		try{
		  JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
		  jms=home.create();
		}catch(Exception e){throw new EngineException(e.getMessage());}
		String type=(String)n.getType();
		String projectName=type.substring(type.lastIndexOf(":")+1);
		jms.sendProjectEvent(hero.util.EventConstants.NODEDEADLINE,projectName,(String)n.getMessage(),"Timer");
	}catch(Exception ex){ex.printStackTrace();}*/
	}
	
	public void doStop() throws ServiceException {
		if (mbeanServer != null) {
			try {
				mbeanServer.unregisterMBean(new ObjectName(SERVICE_NAME));
			} catch (Exception e) {
				throw new ServiceException("Cannot stop the TimerReceiver Service (JMX).", e);
			}
		}
		System.out.println("TimerReceiver :: Stop Service");
	}
}
